#ifndef INTERACTABLEGROUNDOBJECT_HPP_INCLUDED
#define INTERACTABLEGROUNDOBJECT_HPP_INCLUDED

#include "../../Core/Classes/GroundObject.hpp"
#include "../../Core/Classes/ObjectInfo.hpp"
#include "../../Core/Types/Tile.hpp"
#include "../../Core/Types/Point.hpp"
#include "../../Core/Types/Box.hpp"
#include "../../Core/Types/Convex.hpp"
#include "../../Core/Input.hpp"
#include "../../Game/Tools/Camera.hpp"
#include <cstdint>

namespace Interactable
{
    class GroundObject : public Internal::GroundObject
    {
        public:
            GroundObject(const Internal::GroundObject& G);
            GroundObject(const GroundObject& G);

            std::int32_t GetID() const;
            std::string GetName() const;
            Internal::ObjectInfo GetInfo() const;

            std::vector<Point> GetModel() const;
            Convex GetConvex() const;
            Box GetBox() const;
            Point GetPoint() const;
            Point GetPoint(double PointGenAccuracy) const;

            double GetVisibility() const;
            Tile GetTile() const;
            Tile GetReachableTile() const;
            bool IsReachable() const;
            bool RotateTo(std::int32_t Angle = 0) const;
            bool RotateTo(Camera::COMPASS_DIRECTION Direction) const;

            bool Hover() const;
            bool Interact(const Button& B = BUTTON_LEFT) const;
            bool Interact(const std::string& Option, bool CheckTarget = true) const;
            bool Interact(const std::vector<std::string>& Options, bool CheckTarget = true) const;

    };
}

#endif // INTERACTABLEGROUNDOBJECT_HPP_INCLUDED
