#ifndef INTERACTABLEPLAYER_HPP_INCLUDED
#define INTERACTABLEPLAYER_HPP_INCLUDED

#include "../../Core/Classes/Player.hpp"
#include "../../Core/Classes/PlayerInfo.hpp"
#include "../../Core/Types/Tile.hpp"
#include "../../Core/Types/Point.hpp"
#include "../../Core/Types/Box.hpp"
#include "../../Core/Types/Convex.hpp"
#include "../../Core/Input.hpp"
#include "../../Game/Tools/Camera.hpp"
#include <cstdint>

namespace Interactable
{
    class Player : public Internal::Player
    {
        public:
            Player(const Internal::Player& P);
            Player(const Player& P);

            std::string GetName() const;
            Internal::PlayerInfo GetInfo() const;

            std::vector<Point> GetModel() const;
            Convex GetConvex() const;
            Box GetBox() const;
            Point GetPoint() const;
            Point GetPoint(double PointGenAccuracy) const;

            double GetVisibility() const;
            Tile GetTile() const;
            Tile GetReachableTile() const;
            bool IsReachable() const;
            bool RotateTo(std::int32_t Angle = 0) const;
            bool RotateTo(Camera::COMPASS_DIRECTION Direction) const;

            Internal::Character GetInteracting() const;
            bool Interacting() const;
            bool Interacting(const Internal::Character& C) const;

            bool Hover() const;
            bool Interact(const Button& B = BUTTON_LEFT) const;
            bool Interact(const std::string& Option, bool CheckTarget = true) const;
            bool Interact(const std::vector<std::string>& Options, bool CheckTarget = true) const;

            operator Internal::Player() const;
    };
}


#endif // INTERACTABLEPLAYER_HPP_INCLUDED
