#ifndef INTERACTABLEWALLOBJECT_HPP_INCLUDED
#define INTERACTABLEWALLOBJECT_HPP_INCLUDED

#include "../../Core/Classes/WallObject.hpp"
#include "../../Core/Classes/ObjectInfo.hpp"
#include "../../Core/Types/Tile.hpp"
#include "../../Core/Types/Point.hpp"
#include "../../Core/Types/Box.hpp"
#include "../../Core/Types/Convex.hpp"
#include "../../Core/Input.hpp"
#include "../../Game/Tools/Camera.hpp"
#include <cstdint>

namespace Interactable
{
    class WallObject : public Internal::WallObject
    {
        public:
            WallObject(const Internal::WallObject& W);
            WallObject(const WallObject& W);

            std::int32_t GetID() const;
            std::string GetName() const;
            Internal::ObjectInfo GetInfo() const;

            std::vector<Point> GetModel() const;
            Convex GetConvex() const;
            Box GetBox() const;
            Point GetPoint() const;
            Point GetPoint(double PointGenAccuracy) const;

            double GetVisibility() const;
            Tile GetTile() const;
            Tile GetReachableTile() const;
            bool IsReachable() const;
            bool RotateTo(std::int32_t Angle = 0) const;
            bool RotateTo(Camera::COMPASS_DIRECTION Direction) const;

            bool Hover() const;
            bool Interact(const Button& B = BUTTON_LEFT) const;
            bool Interact(const std::string& Option, bool CheckTarget = true) const;
            bool Interact(const std::vector<std::string>& Options, bool CheckTarget = true) const;

            operator Internal::WallObject() const;
    };
}


#endif // INTERACTABLEWALLOBJECT_HPP_INCLUDED
