#ifndef INVENTORY_HPP_INCLUDED
#define INVENTORY_HPP_INCLUDED

#include "../../../Core/Types/Box.hpp"
#include "../../../Core/Types/Convex.hpp"
#include "../../../Game/Interactable/Item.hpp"
#include <functional>
#include <vector>
#include <cstdint>
#include <string>

/** @addtogroup GameTabs
* @{ */
class Inventory
{
    public:

        static bool IsOpen();
        static bool Open();

        static std::vector<std::int32_t> GetItemIDs();
        static std::vector<std::string> GetItemNames();
        static std::vector<std::int32_t> GetItemAmounts();

        static std::vector<Interactable::Item> GetItems();
        static std::vector<Interactable::Item> GetItems(std::int32_t ID);
        static std::vector<Interactable::Item> GetItems(const std::string& Name);
        static std::vector<Interactable::Item> GetItems(const std::vector<std::int32_t>& IDs);
        static std::vector<Interactable::Item> GetItems(const std::vector<std::string>& Names);
        static std::vector<Interactable::Item> GetItems(const std::function<bool (const Interactable::Item&)>& Filter);

        static Interactable::Item GetItem(std::int32_t ID);
        static Interactable::Item GetItem(const std::string& Name);
        static Interactable::Item GetItem(const std::vector<std::int32_t>& IDs);
        static Interactable::Item GetItem(const std::vector<std::string>& Names);
        static Interactable::Item GetItem(const std::function<bool (const Interactable::Item&)>& Filter);

        static bool IsEmpty();
        static bool IsFull();

        static bool IsItemSelected();
        static bool IsItemSelected(std::int32_t ID);
        static bool IsItemSelected(const std::string& Name);
        static bool IsItemSelected(const Interactable::Item& Item);

        static std::string GetItemSelectedName();

        static std::int32_t Count(std::int32_t ID);
        static std::int32_t Count(const std::string& Name);
        static std::int32_t Count(const Interactable::Item& Item);
        static std::int32_t Count(const std::vector<std::int32_t>& IDs);
        static std::int32_t Count(const std::vector<std::string>& Names);
        static std::int32_t Count(const std::vector<Interactable::Item>& Items);

        static std::int32_t CountOccupied();
        static std::int32_t CountEmpty();

        static std::int32_t GetIndexOf(std::int32_t ID);
        static std::int32_t GetIndexOf(const std::string& Name);
        static std::int32_t GetIndexOf(const std::vector<std::int32_t>& IDs); // Returns first found ID index
        static std::int32_t GetIndexOf(const std::vector<std::string>& Names); // Returns first found Name Index

        static std::vector<std::int32_t> GetIndicesOf(std::int32_t ID);
        static std::vector<std::int32_t> GetIndicesOf(const std::string& Name);
        static std::vector<std::int32_t> GetIndicesOf(const std::vector<std::int32_t>& IDs);
        static std::vector<std::int32_t> GetIndicesOf(const std::vector<std::string>& Names);

        static std::vector<Box> GetSlotBoxes();
        static std::vector<Box> GetBoxesOf(std::int32_t ID);
        static std::vector<Box> GetBoxesOf(const std::string& Name);

        static bool Contains(std::int32_t ID);
        static bool Contains(const std::string& Name);
        static bool Contains(const Interactable::Item& Item);
        static bool Contains(const std::vector<std::int32_t>& IDs); // true if all items are found at least once
        static bool Contains(const std::vector<std::string>& Names); // true if all items are found at least once
        static bool Contains(const std::vector<Interactable::Item>& Items);

        static bool ContainsAny(const std::vector<std::int32_t>& IDs);
        static bool ContainsAny(const std::vector<std::string>& Names);
        static bool ContainsAny(const std::vector<Interactable::Item>& Items);

        static bool ContainsOnly(std::int32_t ID);
        static bool ContainsOnly(const std::string& Name);
        static bool ContainsOnly(const Interactable::Item& Item);
        static bool ContainsOnly(const std::vector<std::int32_t>& IDs);
        static bool ContainsOnly(const std::vector<std::string>& Names);
        static bool ContainsOnly(const std::vector<Interactable::Item>& Items);

        static bool Use(std::int32_t ID, std::int32_t ID2 = -1, bool UseSecondFirst = false);
        static bool Use(const std::string& Name, const std::string& Name2 = "", bool UseSecondFirst = false);
        static bool Use(const Interactable::Item& Item, const Interactable::Item& Item2 = Interactable::Item(), bool UseSecondFirst = false);

        static bool Equip(std::int32_t ID);
        static bool Equip(const std::string& Name);
        static bool Equip(const Interactable::Item& Item);

        static bool Drop(std::int32_t ID, bool AllowShiftClick = true);
        static bool Drop(const std::string& Name, bool AllowShiftClick = true);
        static bool Drop(const Interactable::Item& Item, bool AllowShiftClick = true);
};

/** @} */

#endif // INVENTORY_HPP_INCLUDED
